# BSD 2-Clause License
#
# Copyright (c) 2018, Elia El Lazkani
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
Local mode dependencies:
  - weechat
  - dbus-python
  - notify2

Remote mode dependencies:
  - weechat

Server mode dependencies:
  - dbus-python
  - notify2

This script acts as both server and weechat plugin. One can configure it to run
locally on the box in which case it will use *notify2* to send notifications
through dbus to the user. Or, one can run it in remote mode where both
the _host_ and the _port_ need to be configured to send notifications to the server.

If you are running weechat in an ssh session, you can do the following:
> ssh -R 5431:localhost:5431 user@host

The above will ssh tunnel port 5431 between guest and host and notifications can be
sent to the server running locally on your machine.

To run the server, do the following:
> python weenotify.py -s

One can get the full script help print with:
> python weenotify.py -h
"""

import json
import socket
import argparse

try:
    import notify2 as Notify
    import dbus
    notify_imported = True
    dbus_imported = True
except ImportError:
    try:
        import dbus
        dbus_imported = True
    except ImportError:
        dbus_imported = False
    notify_imported = False

try:
    import weechat as w
    weechat_imported = True
except ImportError:
    weechat_imported = False

SCRIPT_NAME = "weenotify"
SCRIPT_AUTHOR = "Elia El Lazkani <elia.el.lazkani@gmail.com>"
SCRIPT_VERSION = "0.4"
SCRIPT_LICENSE = "BSD-2-Clause"
SCRIPT_DESCRIPTION = "Plugin/Server to send/receive notifications and display them"

def listener(sockt):
    """ Method to handle incoming data from client """
    conn, addr = sockt.accept()
    try:
        data = ""
        _partial = conn.recv(1024).decode('utf-8')
        while _partial:
            data += _partial
            _partial = conn.recv(1024).decode('utf-8')
    except Exception:
        conn.close()
        return

    if data:
        print("{}: {}".format(addr, data.strip('\n')))
        notify(json.loads(data.strip()))

def server(host, port):
    """ Method to run the server in a loop """
    print("Starting server...")
    host = host if host else ''
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.bind((host, port))
    s.listen(5)
    if host:
        print("Server listening on {}:{}...".format(host, port))
    else:
        print("Server listening locally on port {}...".format(port))

    try:
        while True:
            listener(s)
    except KeyboardInterrupt:
        print("Shutting down server...")
    finally:
        s.close()

def notify(data):
    """ Method to notify the user through dbus """
    if data['prefix'] != '--':
        try:
            msg = "{} => {}: {}".format(data['buffer'],
                                        data['prefix'],
                                        data['message'])
        except UnicodeEncodeError:
            # Yey to python2
            msg = "{} => {}: {}".format(data['buffer'],
                                        data['prefix'],
                                        data['message'].encode('utf-8'))
    else:
        try:
            msg = "{} => {}".format(data['buffer'],
                                    data['message'])
        except UnicodeEncodeError:
            # Woohoo python2
            msg = "{} => {}".format(data['buffer'],
                                    data['message'].encode('utf-8'))
    Notify.init(SCRIPT_NAME)
    notification = Notify.Notification(data['type'], msg)
    try:
        notification.show()
    except dbus.exceptions.DBusException as e:
        print(e)
        pass

def local_notify(data):
    """ Method to send notification locally on the host """
    if notify_imported:
        notify(data)
    else:
        if dbus_imported:
            w.prnt("", "notify2 could not be imported, disabling {}"
                .format(SCRIPT_NAME))
        else:
            w.prnt("", "notify2 could not be imported due to missing dbus-python,"
                   "disabling {}".format(SCRIPT_NAME))
        w.config_set_plugin('enable', 'off')

def send(host, port, msg):
    """ Method to send data to the server for notification display """
    c = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        c.connect((host, port))
    except Exception:
        c.close()
        return
    message = "{}".format(json.dumps(msg))
    message = str.encode(message, 'utf-8')
    c.sendall(message)
    c.close()

def on_notify(data, buffer, date, tags, displayed,
              highlight, prefix, message):
    """ Method to take action on client notification """
    enable = w.config_get_plugin('enable')
    if enable.lower() == 'on':
        data = weechat_parser(data, buffer, date, tags, displayed,
                              highlight, prefix, message)
        if data:
            mode = w.config_get_plugin('mode')
            if mode.lower() == 'remote':
                host = w.config_get_plugin('host')
                port = int(w.config_get_plugin('port'))
                send(host, port, data)
            else:
                local_notify(data)
    return w.WEECHAT_RC_OK

def weechat_parser(data, buffer, date, tags, displayed,
                   highlight, prefix, message):
    """ Method to parte data coming from weechat and take action on it """
    buffer_name = w.buffer_get_string(buffer, 'short_name')
    if "NOTICE" in data and buffer_name != "highmon":
        if buffer_name == prefix:
            buffer_name = data
        return { 'buffer': buffer_name,
                 'type': data,
                 'prefix': prefix,
                 'message': message }
    elif "PRIVMSG" in data and "irc_notice" not in tags:
        return { 'buffer': buffer_name,
                 'type': data,
                 'prefix': prefix,
                 'message': message }
    elif "MSG" in data and int(highlight) and \
         "irc_notice" not in tags:
        return { 'buffer': buffer_name,
                 'type': data,
                 'prefix': prefix,
                 'message': message }

def client():
    """ Method to register the plugin and hook into weechat """
    settings = {
        'enable': {
            'description': 'Enable/Disable notifications.',
            'values': ['off', 'on'],
            'default': 'off'
        },
        'mode': {
            'description': 'Set whether notifications need to be'
            'sent locally or to an external server.',
            'values': ['local', 'remote'],
            'default': 'local'
        },
        'host': {
            'description': 'Set the server host to send notifications to.',
            'values': None,
            'default': 'localhost'
        },
        'port': {
            'description': 'Set the server port to use to send notifcation.',
            'values': None,
            'default': '5431'
        }
    }

    if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                  SCRIPT_DESCRIPTION, '', ''):
        for option, value in settings.items():
            if not w.config_is_set_plugin(option):
                w.config_set_plugin(option, value['default'])
            if value.get('values', None):
                w.config_set_desc_plugin(option, '{} (values: [{}], default: {})'.format(
                    value['description'], '/'.join(value['values']), value['default']))
            else:
                w.config_set_desc_plugin(option, '{} (default: {})'.format(
                    value['description'], value['default']))
        w.hook_print('', 'notify_message', '', 1, 'on_notify', 'MSG')
        w.hook_print('', 'notify_private', '', 1, 'on_notify', 'PRIVMSG')
        w.hook_print('', 'irc_notice', '', 1, 'on_notify', 'NOTICE')

def argument_parse():
    """ Method to parse command line arguments """
    parser = argparse.ArgumentParser(
        description=SCRIPT_DESCRIPTION)

    parser.add_argument(
        '-s', '--server', action='store_true',
        help='Run in server mode.')
    parser.add_argument(
        '-H', '--host', type=str, default='',
        help='The host/IP to bind to.')
    parser.add_argument(
        '-p', '--port', type=int, default=5431,
        help='The port to listen to.')
    parser.add_argument(
        '-V', '--version', action='version',
        version='%(prog)s {}'.format(SCRIPT_VERSION),
        help='Prints version.')
    return parser

if __name__ == '__main__':
    parser = argument_parse()
    args = parser.parse_args()
    if args.server:
        if notify_imported:
            server(args.host, args.port)
        else:
            if dbus_imported:
                print("notify2 could not be imported")
            else:
                print("notify2 was not imported due to missing dbus-python")
    else:
        if weechat_imported:
            client()
        else:
            print("weechat could not be imported, make sure weechat is running this")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Adam Spiers <weechat@adamspiers.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# (this script probably requires WeeChat 0.3.0 or newer)
#
# History:
#
# 2013-01-07 Adam Spiers <weechat@adamspiers.org>
#     version 0.2: bugfixes
# 2013-01-06 Adam Spiers <weechat@adamspiers.org>
#     version 0.1: initial release

# http://www.weechat.org/files/doc/stable/weechat_scripting.en.html

import weechat as w

SCRIPT_NAME    = "toggle_highlight"
SCRIPT_AUTHOR  = "Adam Spiers <weechat@adamspiers.org>"
SCRIPT_VERSION = "0.2"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Toggles notifications of normal messages for the current buffer"

SCRIPT_COMMAND = "highlight"

SILENCE_DESCR  = "notifications for normal messages silenced"
NOTIFY_DESCR   = "notifications for normal messages enabled"

settings = {
}

def show_message(buf, msg):
    w.prnt(buf, '[%s] %s' % (SCRIPT_COMMAND, msg))

def get_setting_name(buf):
    bufchan = w.buffer_get_string(buf, 'localvar_channel')
    bufserv = w.buffer_get_string(buf, 'localvar_server')
    return "weechat.notify.irc.%s.%s" % (bufserv, bufchan)

def is_current_buffer_highlight_set(buf):
    return w.config_string(w.config_get(get_setting_name(buf))) == 'highlight'

def display_current(buf):
    if not ensure_channel_buffer(buf):
        return

    if is_current_buffer_highlight_set(buf):
        show_message(buf, SILENCE_DESCR)
    else:
        show_message(buf, NOTIFY_DESCR)

def get_buffers_list():
    buffers = w.config_get_plugin('buffers')
    if buffers == '':
        return []
    else:
        return buffers.split(',')

def ensure_channel_buffer(buf):
    buftype = w.buffer_get_string(buf, 'localvar_type')

    if buftype != 'channel':
        show_message(
            buf,
            "current buffer type is %s not a channel!\n"
            "run this in a channel buffer." % buftype)
        return False

    return True

def set_highlight(buf, unset=False):
    if not ensure_channel_buffer(buf):
        return

    if unset:
        command = "/unset %s" % get_setting_name(buf)
        message = NOTIFY_DESCR
    else:
        command = "/set %s highlight" % get_setting_name(buf)
        message = SILENCE_DESCR

    w.command('', command)
    show_message(buf, message)

def set_highlight_cmd_cb(data, buf, args):
    ''' Command /highlight '''
    if args == '':
        display_current(buf)
    else:
        if args == 'toggle':
            if is_current_buffer_highlight_set(buf):
                set_highlight(buf, unset=True)
            else:
                set_highlight(buf)
        elif args == 'unset':
            set_highlight(buf, unset=True)
        elif args == 'set':
            set_highlight(buf)
        else:
            show_message(buf,
                         "Invalid arguments %s to /%s" %
                         (args, SCRIPT_COMMAND))

    return w.WEECHAT_RC_OK

if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
    for option, default_value in settings.items():
        if not w.config_is_set_plugin(option):
            w.config_set_plugin(option, default_value)

    w.hook_command(SCRIPT_COMMAND,
                   "Toggle / set highlight notification level on current buffer",
                   "[unset|set|toggle]",
                   "   set : silence notifications for normal messages in current channel\n"
                   " unset : enable notifications for normal messages in current channel\n"
                   "toggle : toggle the setting\n\n"
                   "with no parameter, shows the current state",
                   "unset|set|toggle",
                   "set_highlight_cmd_cb", "")



# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 by xt <xt@bash.no>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# This script colors nicks in IRC channels in the actual message
# not just in the prefix section.
#
#
# History:
# 2018-04-06: Joey Pabalinas <joeypabalinas@gmail.com>
#   version 26: fix freezes with too many nicks in one line
# 2018-03-18: nils_2
#   version 25: fix unable to run function colorize_config_reload_cb()
# 2017-06-20: lbeziaud <louis.beziaud@ens-rennes.fr>
#   version 24: colorize utf8 nicks
# 2017-03-01, arza <arza@arza.us>
#   version 23: don't colorize nicklist group names
# 2016-05-01, Simmo Saan <simmo.saan@gmail.com>
#   version 22: invalidate cached colors on hash algorithm change
# 2015-07-28, xt
#   version 21: fix problems with nicks with commas in them
# 2015-04-19, xt
#   version 20: fix ignore of nicks in URLs
# 2015-04-18, xt
#   version 19: new option ignore nicks in URLs
# 2015-03-03, xt
#   version 18: iterate buffers looking for nicklists instead of servers
# 2015-02-23, holomorph
#   version 17: fix coloring in non-channel buffers (#58)
# 2014-09-17, holomorph
#   version 16: use weechat config facilities
#               clean unused, minor linting, some simplification
# 2014-05-05, holomorph
#   version 15: fix python2-specific re.search check
# 2013-01-29, nils_2
#   version 14: make script compatible with Python 3.x
# 2012-10-19, ldvx
#   version 13: Iterate over every word to prevent incorrect colorization of
#               nicks. Added option greedy_matching.
# 2012-04-28, ldvx
#   version 12: added ignore_tags to avoid colorizing nicks if tags are present
# 2012-01-14, nesthib
#   version 11: input_text_display hook and modifier to colorize nicks in input bar
# 2010-12-22, xt
#   version 10: hook config option for updating blacklist
# 2010-12-20, xt
#   version 0.9: hook new config option for weechat 0.3.4
# 2010-11-01, nils_2
#   version 0.8: hook_modifier() added to communicate with rainbow_text
# 2010-10-01, xt
#   version 0.7: changes to support non-irc-plugins
# 2010-07-29, xt
#   version 0.6: compile regexp as per patch from Chris quigybo@hotmail.com
# 2010-07-19, xt
#   version 0.5: fix bug with incorrect coloring of own nick
# 2010-06-02, xt
#   version 0.4: update to reflect API changes
# 2010-03-26, xt
#   version 0.3: fix error with exception
# 2010-03-24, xt
#   version 0.2: use ignore_channels when populating to increase performance.
# 2010-02-03, xt
#   version 0.1: initial (based on ruby script by dominikh)
#
# Known issues: nicks will not get colorized if they begin with a character
# such as ~ (which some irc networks do happen to accept)

import weechat
import re
w = weechat

SCRIPT_NAME    = "colorize_nicks"
SCRIPT_AUTHOR  = "xt <xt@bash.no>"
SCRIPT_VERSION = "26"
SCRIPT_LICENSE = "GPL"
SCRIPT_DESC    = "Use the weechat nick colors in the chat area"

# Based on the recommendations in RFC 7613. A valid nick is composed
# of anything but " ,*?.!@".
VALID_NICK = r'([@~&!%+-])?([^\s,\*?\.!@]+)'
valid_nick_re = re.compile(VALID_NICK)
ignore_channels = []
ignore_nicks = []

# Dict with every nick on every channel with its color as lookup value
colored_nicks = {}

CONFIG_FILE_NAME = "colorize_nicks"

# config file and options
colorize_config_file = ""
colorize_config_option = {}

def colorize_config_init():
    '''
    Initialization of configuration file.
    Sections: look.
    '''
    global colorize_config_file, colorize_config_option
    colorize_config_file = weechat.config_new(CONFIG_FILE_NAME,
                                              "", "")
    if colorize_config_file == "":
        return

    # section "look"
    section_look = weechat.config_new_section(
        colorize_config_file, "look", 0, 0, "", "", "", "", "", "", "", "", "", "")
    if section_look == "":
        weechat.config_free(colorize_config_file)
        return
    colorize_config_option["blacklist_channels"] = weechat.config_new_option(
        colorize_config_file, section_look, "blacklist_channels",
        "string", "Comma separated list of channels", "", 0, 0,
        "", "", 0, "", "", "", "", "", "")
    colorize_config_option["blacklist_nicks"] = weechat.config_new_option(
        colorize_config_file, section_look, "blacklist_nicks",
        "string", "Comma separated list of nicks", "", 0, 0,
        "so,root", "so,root", 0, "", "", "", "", "", "")
    colorize_config_option["min_nick_length"] = weechat.config_new_option(
        colorize_config_file, section_look, "min_nick_length",
        "integer", "Minimum length nick to colorize", "",
        2, 20, "", "", 0, "", "", "", "", "", "")
    colorize_config_option["colorize_input"] = weechat.config_new_option(
        colorize_config_file, section_look, "colorize_input",
        "boolean", "Whether to colorize input", "", 0,
        0, "off", "off", 0, "", "", "", "", "", "")
    colorize_config_option["ignore_tags"] = weechat.config_new_option(
        colorize_config_file, section_look, "ignore_tags",
        "string", "Comma separated list of tags to ignore; i.e. irc_join,irc_part,irc_quit", "", 0, 0,
        "", "", 0, "", "", "", "", "", "")
    colorize_config_option["greedy_matching"] = weechat.config_new_option(
        colorize_config_file, section_look, "greedy_matching",
        "boolean", "If off, then use lazy matching instead", "", 0,
        0, "on", "on", 0, "", "", "", "", "", "")
    colorize_config_option["match_limit"] = weechat.config_new_option(
        colorize_config_file, section_look, "match_limit",
        "integer", "Fall back to lazy matching if greedy matches exceeds this number", "",
        20, 1000, "", "", 0, "", "", "", "", "", "")
    colorize_config_option["ignore_nicks_in_urls"] = weechat.config_new_option(
        colorize_config_file, section_look, "ignore_nicks_in_urls",
        "boolean", "If on, don't colorize nicks inside URLs", "", 0,
        0, "off", "off", 0, "", "", "", "", "", "")

def colorize_config_read():
    ''' Read configuration file. '''
    global colorize_config_file
    return weechat.config_read(colorize_config_file)

def colorize_nick_color(nick, my_nick):
    ''' Retrieve nick color from weechat. '''
    if nick == my_nick:
        return w.color(w.config_string(w.config_get('weechat.color.chat_nick_self')))
    else:
        return w.info_get('irc_nick_color', nick)

def colorize_cb(data, modifier, modifier_data, line):
    ''' Callback that does the colorizing, and returns new line if changed '''

    global ignore_nicks, ignore_channels, colored_nicks


    full_name = modifier_data.split(';')[1]
    channel = '.'.join(full_name.split('.')[1:])

    buffer = w.buffer_search('', full_name)
    # Check if buffer has colorized nicks
    if buffer not in colored_nicks:
        return line

    if channel and channel in ignore_channels:
        return line

    min_length = w.config_integer(colorize_config_option['min_nick_length'])
    reset = w.color('reset')

    # Don't colorize if the ignored tag is present in message
    tags_line = modifier_data.rsplit(';')
    if len(tags_line) >= 3:
        tags_line = tags_line[2].split(',')
        for i in w.config_string(colorize_config_option['ignore_tags']).split(','):
            if i in tags_line:
                return line

    for words in valid_nick_re.findall(line):
        nick = words[1]
        # Check that nick is not ignored and longer than minimum length
        if len(nick) < min_length or nick in ignore_nicks:
            continue

        # If the matched word is not a known nick, we try to match the
        # word without its first or last character (if not a letter).
        # This is necessary as "foo:" is a valid nick, which could be
        # adressed as "foo::".
        if nick not in colored_nicks[buffer]:
            if not nick[-1].isalpha() and not nick[0].isalpha():
                if nick[1:-1] in colored_nicks[buffer]:
                    nick = nick[1:-1]
            elif not nick[0].isalpha():
                if nick[1:] in colored_nicks[buffer]:
                    nick = nick[1:]
            elif not nick[-1].isalpha():
                if nick[:-1] in colored_nicks[buffer]:
                    nick = nick[:-1]

        # Check that nick is in the dictionary colored_nicks
        if nick in colored_nicks[buffer]:
            nick_color = colored_nicks[buffer][nick]

            try:
                # Let's use greedy matching. Will check against every word in a line.
                if w.config_boolean(colorize_config_option['greedy_matching']):
                    cnt = 0
                    limit = w.config_integer(colorize_config_option['match_limit'])

                    for word in line.split():
                        cnt += 1
                        assert cnt < limit
                        #  if cnt > limit:
                            #  raise RuntimeError('Exceeded colorize_nicks.look.match_limit.');

                        if w.config_boolean(colorize_config_option['ignore_nicks_in_urls']) and \
                              word.startswith(('http://', 'https://')):
                            continue

                        if nick in word:
                            # Is there a nick that contains nick and has a greater lenght?
                            # If so let's save that nick into var biggest_nick
                            biggest_nick = ""
                            for i in colored_nicks[buffer]:
                                cnt += 1
                                assert cnt < limit

                                if nick in i and nick != i and len(i) > len(nick):
                                    if i in word:
                                        # If a nick with greater len is found, and that word
                                        # also happens to be in word, then let's save this nick
                                        biggest_nick = i
                            # If there's a nick with greater len, then let's skip this
                            # As we will have the chance to colorize when biggest_nick
                            # iterates being nick.
                            if len(biggest_nick) > 0 and biggest_nick in word:
                                pass
                            elif len(word) < len(biggest_nick) or len(biggest_nick) == 0:
                                new_word = word.replace(nick, '%s%s%s' % (nick_color, nick, reset))
                                line = line.replace(word, new_word)

                # Switch to lazy matching
                else:
                    raise AssertionError

            except AssertionError:
                # Let's use lazy matching for nick
                nick_color = colored_nicks[buffer][nick]
                # The two .? are in case somebody writes "nick:", "nick,", etc
                # to address somebody
                regex = r"(\A|\s).?(%s).?(\Z|\s)" % re.escape(nick)
                match = re.search(regex, line)
                if match is not None:
                    new_line = line[:match.start(2)] + nick_color+nick+reset + line[match.end(2):]
                    line = new_line

    return line

def colorize_input_cb(data, modifier, modifier_data, line):
    ''' Callback that does the colorizing in input '''

    global ignore_nicks, ignore_channels, colored_nicks

    min_length = w.config_integer(colorize_config_option['min_nick_length'])

    if not w.config_boolean(colorize_config_option['colorize_input']):
        return line

    buffer = w.current_buffer()
    # Check if buffer has colorized nicks
    if buffer not in colored_nicks:
        return line

    channel = w.buffer_get_string(buffer, 'name')
    if channel and channel in ignore_channels:
        return line

    reset = w.color('reset')

    for words in valid_nick_re.findall(line):
        nick = words[1]
        # Check that nick is not ignored and longer than minimum length
        if len(nick) < min_length or nick in ignore_nicks:
            continue
        if nick in colored_nicks[buffer]:
            nick_color = colored_nicks[buffer][nick]
            line = line.replace(nick, '%s%s%s' % (nick_color, nick, reset))

    return line

def populate_nicks(*args):
    ''' Fills entire dict with all nicks weechat can see and what color it has
    assigned to it. '''
    global colored_nicks

    colored_nicks = {}

    buffers = w.infolist_get('buffer', '', '')
    while w.infolist_next(buffers):
        buffer_ptr = w.infolist_pointer(buffers, 'pointer')
        my_nick = w.buffer_get_string(buffer_ptr, 'localvar_nick')
        nicklist = w.infolist_get('nicklist', buffer_ptr, '')
        while w.infolist_next(nicklist):
            if buffer_ptr not in colored_nicks:
                colored_nicks[buffer_ptr] = {}

            if w.infolist_string(nicklist, 'type') != 'nick':
                continue

            nick = w.infolist_string(nicklist, 'name')
            nick_color = colorize_nick_color(nick, my_nick)

            colored_nicks[buffer_ptr][nick] = nick_color

        w.infolist_free(nicklist)

    w.infolist_free(buffers)

    return w.WEECHAT_RC_OK

def add_nick(data, signal, type_data):
    ''' Add nick to dict of colored nicks '''
    global colored_nicks

    # Nicks can have , in them in some protocols
    splitted = type_data.split(',')
    pointer = splitted[0]
    nick = ",".join(splitted[1:])
    if pointer not in colored_nicks:
        colored_nicks[pointer] = {}

    my_nick = w.buffer_get_string(pointer, 'localvar_nick')
    nick_color = colorize_nick_color(nick, my_nick)

    colored_nicks[pointer][nick] = nick_color

    return w.WEECHAT_RC_OK

def remove_nick(data, signal, type_data):
    ''' Remove nick from dict with colored nicks '''
    global colored_nicks

    # Nicks can have , in them in some protocols
    splitted = type_data.split(',')
    pointer = splitted[0]
    nick = ",".join(splitted[1:])

    if pointer in colored_nicks and nick in colored_nicks[pointer]:
        del colored_nicks[pointer][nick]

    return w.WEECHAT_RC_OK

def update_blacklist(*args):
    ''' Set the blacklist for channels and nicks. '''
    global ignore_channels, ignore_nicks
    ignore_channels = w.config_string(colorize_config_option['blacklist_channels']).split(',')
    ignore_nicks = w.config_string(colorize_config_option['blacklist_nicks']).split(',')
    return w.WEECHAT_RC_OK

if __name__ == "__main__":
    if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                  SCRIPT_DESC, "", ""):
        colorize_config_init()
        colorize_config_read()

        # Run once to get data ready
        update_blacklist()
        populate_nicks()

        w.hook_signal('nicklist_nick_added', 'add_nick', '')
        w.hook_signal('nicklist_nick_removed', 'remove_nick', '')
        w.hook_modifier('weechat_print', 'colorize_cb', '')
        # Hook config for changing colors
        w.hook_config('weechat.color.chat_nick_colors', 'populate_nicks', '')
        w.hook_config('weechat.look.nick_color_hash', 'populate_nicks', '')
        # Hook for working togheter with other scripts (like colorize_lines)
        w.hook_modifier('colorize_nicks', 'colorize_cb', '')
        # Hook for modifying input
        w.hook_modifier('250|input_text_display', 'colorize_input_cb', '')
        # Hook for updating blacklist (this could be improved to use fnmatch)
        weechat.hook_config('%s.look.blacklist*' % SCRIPT_NAME, 'update_blacklist', '')



# -*- coding: utf-8 -*-
# Copyright (c) 2014 by Vlad Stoica <stoica.vl@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# History
# 02-07-2014 - Vlad Stoica
# Initial script

try:
    import weechat
    import HTMLParser
    import_error = 0
except ImportError:
    import_error = 1

SCRIPT_NAME = "smile"
SCRIPT_AUTHOR = "Vlad Stoica <stoica.vl@gmail.com>"
SCRIPT_VERSION = "0.1"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = "Prints a random ascii smiley."

cmd_hook_process = ""
cmd_buffer       = ""
cmd_stdout       = ""
cmd_stderr       = ""

def smile_cmd(data, buffer, args):
    global cmd_hook_process, cmd_buffer, cmd_stdout, cmd_stderr
    if cmd_hook_process != "":
        weechat.prnt(buffer, "Already trying to grab a smiley.")
        return weechat.WEECHAT_RC_OK
    cmd_buffer = buffer
    cmd_stdout = ""
    cmd_stderr = ""
    cmd_hook_process = weechat.hook_process("url:http://dominick.p.elu.so/fun/kaomoji/get.php", 10000, "smile_cb", "")
    return weechat.WEECHAT_RC_OK

def smile_cb(data, command, rc, stdout, stderr):
    global cmd_hook_process, cmd_buffer, cmd_stdout, cmd_stderr
    cmd_stdout += stdout
    cmd_stderr += stderr
    if int(rc) >= 0:
        if cmd_stderr != "":
            weechat.prnt(cmd_buffer, "%s" % cmd_stderr)
        if cmd_stdout != "":
            h = HTMLParser.HTMLParser()
            sm = h.unescape(cmd_stdout)
            weechat.command(cmd_buffer, sm.encode('utf-8'))
        cmd_hook_process = ""
    return weechat.WEECHAT_RC_OK

if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
    if import_error:
        weechat.prnt("", "You need to run this script inside weechat.")
    weechat.hook_command(SCRIPT_NAME, SCRIPT_DESC, "༼つ ◕_◕ ༽つ", "", "", "smile_cmd", "")



# -*- coding: utf-8 -*-
#
# UrlGrab, for weechat version >= 0.3.0
#
#   Listens to all channels for URLs, collects them in a list, and launches
#   them in your favourite web server on the local host or a remote server.
#   Copies url to X11 clipboard via xsel
#      (http://www.vergenet.net/~conrad/software/xsel)
#
# Usage:
#
#   The /url command provides access to all UrlGrab functions.  Run
#   '/help url' for complete command usage.
#
#   In general, use '/url list' to list the entire url list for the current
#   channel, and '/url <n>' to launch the nth url in the list.  For
#   example, to launch the first (and most-recently added) url in the list,
#   you would run '/url 1'
#
#   From the server window, you must specify a specific channel for the
#   list and launch commands, for example:
#     /url list weechat
#     /url 3 weechat
#
# Configuration:
#
#   The '/url set' command lets you get and set the following options:
#
#   historysize
#     The maximum number of URLs saved per channel.  Default is 10
#
#   method
#     Must be one of 'local' or 'remote' - Defines how URLs are launched by
#     the script.  If 'local', the script will run 'localcmd' on the host.
#     If 'remote', the script will run 'remotessh remotehost remotecmd' on
#     the local host which should normally use ssh to connect to another
#     host and run the browser command there.
#
#   localcmd
#     The command to run on the local host to launch URLs in 'local' mode.
#     The string '%s' will be replaced with the URL.  The default is
#     'firefox %s'.
#
#   remotessh
#     The command (and arguments) used to connect to the remote host for
#     'remote' mode.  The default is 'ssh -x' which will connect as the
#     current username via ssh and disable X11 forwarding.
#
#   remotehost
#     The remote host to which we will connect in 'remote' mode.  For ssh,
#     this can just be a hostname or 'user@host' to specify a username
#     other than your current login name.  The default is 'localhost'.
#
#   remotecmd
#     The command to execute on the remote host for 'remote' mode.  The
#     default is 'bash -c "DISPLAY=:0.0 firefox '%s'"'  Which runs bash, sets
#     up the environment to display on the remote host's main X display,
#     and runs firefox.  As with 'localcmd', the string '%s' will be
#     replaced with the URL.
#
#   cmdoutput
#     The file where the command output (if any) is saved.  Overwritten
#     each time you launch a new URL.  Default is ~/.weechat/urllaunch.log
#
#   default
#     The command that will be run if no arguemnts to /url are given.
#     Default is show
#
# Requirements:
#
#  - Designed to run with weechat version 0.3 or better.
#      http://www.weechat.org/
#
# Acknowlegements:
#
#  - Based on an earlier version called 'urlcollector.py' by 'kolter' of
#    irc.freenode.net/#weechat Honestly, I just cleaned up the code a bit and
#    made the settings a little more useful (to me).
#
#  - With changes by Leonid Evdokimov (weechat at darkk dot net another dot ru):
#    http://darkk.net.ru/weechat/urlgrab.py
#    v1.1:  added better handling of dead zombie-childs
#           added parsing of private messages
#           added default command setting
#           added parsing of scrollback buffers on load
#    v1.2:  `historysize` was ignored
#
#  - With changes by ExclusivE (exclusive_tm at mail dot ru):
#    v1.3: X11 clipboard support
#
#  - V1.4 Just ported it over to weechat 0.2.7  drubin AT smartcube dot co dot za
#  - V1.5  1) I created a logging feature for urls, Time, Date, buffer, and url.
#           2) Added selectable urls support, similar to the iset plugin (Thanks FlashCode)
#           3) Colors/formats are configuarable.
#           4) browser now uses hook_process (Please test with remote clients)
#           5) Added /url open http://url.com functionality
#           6) Changed urls detection to use regexpressions so should be much better
#                Thanks to xt of #weechat bassed on on urlbar.py
#  - V1.6 FlashCode <flashcode@flashtux.org>: Increase timeout for hook_process
#         (from 1 second to 1 minute)
#  - V1.7 FlashCode <flashcode@flashtux.org>: Update WeeChat site
#  - V1.8 drubin <drubin [at] smartcube . co.za>:
#           - Changed remote cmd to be single option
#           - Added scrolling on up and down arrow keys for /url show
#           - Changed remotecmd to include options with public/private keys password auth doesn't work
#  - V1.9 Specimen <spinifer [at] gmail . com>:
#           - Changed the default command when /url is run with no arguments to 'show'
#           - Removed '/url help' command, because /help <command> is the standard way
#  - V2.0 Xilov: replace "/url help" by "/help url"
#  - V2.1 nand: Changed default: firefox %s to firefox '%s' (localcmd)
#  - V2.2 Sébastien Helleu <flashcode@flashtux.org>: fix reload of config file
#  - V2.3 nand: Allowed trailing )s for unmatched (s in URLs
#  - V2.4 nand: Escaped URLs via URL-encoding instead of shell escaping, fixes '
#  - V2.5 nand: Fixed some URLs that got incorrectly mangled by escaping
#  - V2.6 nesthib: Fixed escaping of "="
#                  Added missing quotes in default parameter (firefox '%s')
#                  Removed the mix of tabs and spaces in the file indentation
#  - V2.7 dobbymoodge <john.w.lamb [at] gmail . com>
#                     ( https://github.com/dobbymoodge/ ):
#           - Added 'copycmd' setting, users can set command to pipe into
#             for '/url copy'
#  - V2.8 Simmo Saan <simmo.saan@gmail.com>:
#           - Changed print hook to ignore filtered lines
#  - V2.9 Dominik Heidler <dominik@heidler.eu>:
#           - Updated script for python3 support (now python2 and 3 are both supported)
#  - V3.0 Sébastien Helleu <flashcode@flashtux.org>:
#           - Fix python 3 compatibility (replace "has_key" by "in")
#
# Copyright (C) 2005 David Rubin <drubin AT smartcube dot co dot za>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
# USA.
#

from __future__ import print_function
import sys
import os
try:
    import weechat
    import_ok = True
except:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: http://www.weechat.org/")
    import_ok = False
import subprocess
import time
try:
    from urllib import quote
except ImportError:
    from urllib.parse import quote
import re
try:
    from UserDict import UserDict
except ImportError:
    from collections import UserDict


octet = r'(?:2(?:[0-4]\d|5[0-5])|1\d\d|\d{1,2})'
ipAddr = r'%s(?:\.%s){3}' % (octet, octet)
# Base domain regex off RFC 1034 and 1738
label = r'[0-9a-z][-0-9a-z]*[0-9a-z]?'
domain = r'%s(?:\.%s)*\.[a-z][-0-9a-z]*[a-z]?' % (label, label)
urlRe = re.compile(r'(\w+://(?:%s|%s)(?::\d+)?(?:/[^\]>\s]*)?)' % (domain, ipAddr), re.I)


SCRIPT_NAME    = "urlgrab"
SCRIPT_AUTHOR  = "David Rubin <drubin [At] smartcube [dot] co [dot] za>"
SCRIPT_VERSION = "3.0"
SCRIPT_LICENSE = "GPL"
SCRIPT_DESC    = "Url functionality Loggin, opening of browser, selectable links"
CONFIG_FILE_NAME= "urlgrab"
SCRIPT_COMMAND = "url"


def stripParens(url):
    return dropChar(')', url.count(')') - url.count('('), url[::-1])[::-1]

def dropChar(c, n, xs):
    if n == 0 or xs == []:
        return xs
    elif xs[0] == c:
        return dropChar(c, n-1, xs[1:])
    else:
        return xs

def urlGrabPrint(message):
    bufferd=weechat.current_buffer()
    if urlGrabSettings['output_main_buffer'] == 1 :
        weechat.prnt("","[%s] %s" % ( SCRIPT_NAME, message ) )
    else :
        weechat.prnt(bufferd,"[%s] %s" % ( SCRIPT_NAME, message ) )

def hashBufferName(bufferp):
    if not weechat.buffer_get_string(bufferp, "short_name"):
        bufferd = weechat.buffer_get_string(bufferp, "name")
    else:
        bufferd = weechat.buffer_get_string(bufferp, "short_name")
    return bufferd

def ug_config_reload_cb(data, config_file):
    """ Reload configuration file. """
    return weechat.config_reload(config_file)

class UrlGrabSettings(UserDict):
    def __init__(self):
        UserDict.__init__(self)
        self.data = {}
        self.config_file = weechat.config_new(CONFIG_FILE_NAME,
                                        "ug_config_reload_cb", "")
        if not self.config_file:
            return

        section_color = weechat.config_new_section(
            self.config_file, "color", 0, 0, "", "", "", "", "", "",
                     "", "", "", "")
        section_default = weechat.config_new_section(
            self.config_file, "default", 0, 0, "", "", "", "", "", "",
                     "", "", "", "")

        self.data['color_buffer']=weechat.config_new_option(
            self.config_file, section_color,
            "color_buffer", "color", "Color to display buffer name", "", 0, 0,
            "red", "red", 0, "", "", "", "", "", "")

        self.data['color_url']=weechat.config_new_option(
            self.config_file, section_color,
            "color_url", "color", "Color to display urls", "", 0, 0,
            "blue", "blue", 0, "", "", "", "", "", "")

        self.data['color_time']=weechat.config_new_option(
            self.config_file, section_color,
            "color_time", "color", "Color to display time", "", 0, 0,
            "cyan", "cyan", 0, "", "", "", "", "", "")

        self.data['color_buffer_selected']=weechat.config_new_option(
            self.config_file, section_color,
            "color_buffer_selected", "color",
            "Color to display buffer selected name", "", 0, 0, "red", "red",
            0, "", "", "", "", "", "")

        self.data['color_url_selected']=weechat.config_new_option(
            self.config_file, section_color,
            "color_url_selected", "color", "Color to display url selected",
             "", 0, 0, "blue", "blue", 0, "", "", "", "", "", "")

        self.data['color_time_selected']=weechat.config_new_option(
            self.config_file, section_color,
            "color_time_selected", "color", "Color to display tim selected",
            "", 0, 0, "cyan", "cyan", 0, "", "", "", "", "", "")

        self.data['color_bg_selected']=weechat.config_new_option(
            self.config_file, section_color,
            "color_bg_selected", "color", "Background for selected row", "", 0, 0,
            "green", "green", 0, "", "", "", "", "", "")

        self.data['historysize']=weechat.config_new_option(
            self.config_file, section_default,
            "historysize", "integer", "Max number of urls to store per buffer",
            "", 0, 999, "10", "10", 0, "", "", "", "", "", "")

        self.data['method']=weechat.config_new_option(
            self.config_file, section_default,
            "method", "string", """Where to launch URLs
            If 'local', runs %localcmd%.
            If 'remote' runs the following command:
            '%remodecmd%'""", "", 0, 0,
            "local", "local", 0, "", "", "", "", "", "")

        self.data['copycmd']=weechat.config_new_option(
            self.config_file, section_default,
            "copycmd", "string",
            "Command to pipe into for 'url copy'. "
            "E.g. to copy into the CLIPBOARD buffer "
            "instead of PRIMARY, you can use 'xsel -b "
            "-i' here.", "", 0, 0,
            "xsel -i", "xsel -i", 0, "", "", "", "", "", "")

        self.data['localcmd']=weechat.config_new_option(
            self.config_file, section_default,
            "localcmd", "string", """Local command to execute""", "", 0, 0,
            "firefox '%s'", "firefox '%s'", 0, "", "", "", "", "", "")

        remotecmd="ssh -x localhost -i ~/.ssh/id_rsa -C \"export DISPLAY=\":0.0\" &&  firefox '%s'\""
        self.data['remotecmd']=weechat.config_new_option(
            self.config_file, section_default,
            "remotecmd", "string", remotecmd, "", 0, 0,
            remotecmd, remotecmd, 0, "", "", "", "", "", "")

        self.data['url_log']=weechat.config_new_option(
            self.config_file, section_default,
            "url_log", "string", """log location""", "", 0, 0,
            "~/.weechat/urls.log", "~/.weechat/urls.log", 0, "", "", "", "", "", "")

        self.data['time_format']=weechat.config_new_option(
            self.config_file, section_default,
            "time_format", "string", """TIme format""", "", 0, 0,
            "%H:%M:%S", "%H:%M:%S", 0, "", "", "", "", "", "")

        self.data['output_main_buffer']=weechat.config_new_option(
            self.config_file, section_default,
            "output_main_buffer", "boolean",
            """Print text to main buffer or current one""", "", 0, 0, "1", "1",
             0, "", "", "", "", "", "")
        weechat.config_read(self.config_file)

    def __getitem__(self, key):
        if key == "historysize":
            return weechat.config_integer(self.data[key])
        elif key == 'output_main_buffer':
            return weechat.config_boolean(self.data[key])
        #elif key.startswith('color'):
        #    return weechat.config_color(self.data[key])
        else:
            return weechat.config_string(self.data[key])

    def prnt(self, name, verbose = True):
        weechat.prnt( ""," %s = %s" % (name.ljust(11), self.data[name]) )

    def prntall(self):
        for key in self.names():
            self.prnt(key, verbose = False)

    def createCmd(self, url):
        str =""
        if self['method'] == 'remote':
            str = self['remotecmd']  % url
        else:
            str =  self['localcmd']  % url
        return str

class UrlGrabber:
    def __init__(self, historysize):
        # init
        self.urls = {}
        self.globalUrls = []
        self.historysize = 5
        # control
        self.setHistorysize(historysize)

    def setHistorysize(self, count):
        if count > 1:
            self.historysize = count

    def getHistorysize(self):
        return self.historysize

    def addUrl(self, bufferp,url ):
        global urlGrabSettings
        self.globalUrls.insert(0,{"buffer":bufferp,
            "url":url, "time":time.strftime(urlGrabSettings["time_format"])})
        #Log urls only if we have set a log path.
        if urlGrabSettings['url_log']:
            try :
                index = self.globalUrls[0]
                logfile = os.path.expanduser(urlGrabSettings['url_log'])
                dout = open(logfile, "a")
                dout.write("%s %s %s\n" % (index['time'],
                                           index['buffer'], index['url']))
                dout.close()
            except :
                print("failed to log url check that %s is valid path" % urlGrabSettings['url_log'])
                pass

        # check for buffer
        if not bufferp in self.urls:
            self.urls[bufferp] = []
        # add url
        if url in self.urls[bufferp]:
            self.urls[bufferp].remove(url)
        self.urls[bufferp].insert(0, url)
        # removing old urls
        while len(self.urls[bufferp]) > self.historysize:
            self.urls[bufferp].pop()

    def hasIndex( self, bufferp, index ):
        return bufferp in self.urls and \
                len(self.url[bufferp]) >= index

    def hasBuffer( self, bufferp ):
        return bufferp in self.urls


    def getUrl(self, bufferp, index):
        url = ""
        if  bufferp in self.urls:
            if len(self.urls[bufferp]) >= index:
                    url = self.urls[bufferp][index-1]
        return url


    def prnt(self, buff):
        found = True
        if buff in self.urls:
            if len(self.urls[buff]) > 0:
                i = 1
                for url in self.urls[buff]:
                    urlGrabPrint("--> " + str(i) + " : " + url)
                    i += 1
            else:
                found = False
        elif buff == "*":
            for b in self.urls.keys():
              self.prnt(b)
        else:
            found = False

        if not found:
            urlGrabPrint(buff + ": no entries")

def urlGrabCheckMsgline(bufferp, message, isdisplayed):
    global urlGrab, max_buffer_length
    if not message or isdisplayed == 0:
        return
    # Ignore output from 'tinyurl.py' and our selfs
    if ( message.startswith( "[AKA] http://tinyurl.com" ) or
        message.startswith("[urlgrab]") ):
        return
    # Check for URLs
    for url in urlRe.findall(message):
        urlGrab.addUrl(bufferp,stripParens(url))
        if max_buffer_length < len(bufferp):
            max_buffer_length = len(bufferp)
        if urlgrab_buffer:
            refresh()


def urlGrabCheck(data, bufferp, uber_empty, tagsn, isdisplayed, ishilight, prefix, message):
    urlGrabCheckMsgline(hashBufferName(bufferp), message, isdisplayed)
    return weechat.WEECHAT_RC_OK

def urlGrabCopy(bufferd, index):
    global urlGrab
    if bufferd == "":
        urlGrabPrint( "No current channel, you must activate one" )
    elif not urlGrab.hasBuffer(bufferd):
        urlGrabPrint("No URL found - Invalid channel")
    else:
        if index <= 0:
            urlGrabPrint("No URL found - Invalid index")
            return
        url = urlGrab.getUrl(bufferd,index)
    if url == "":
        urlGrabPrint("No URL found - Invalid index")
    else:
        try:
            pipe = os.popen(urlGrabSettings['copycmd'],"w")
            pipe.write(url)
            pipe.close()
            urlGrabPrint("Url: %s gone to clipboard." % url)
        except:
            urlGrabPrint("Url: %s failed to copy to clipboard." % url)

def urlGrabOpenUrl(url):
    global urlGrab, urlGrabSettings
    argl = urlGrabSettings.createCmd( quote(url, '/:#%?&+=') )
    weechat.hook_process(argl,60000, "ug_open_cb", "")

def ug_open_cb(data, command, code, out, err):
    #print out
    #print err
    return weechat.WEECHAT_RC_OK


def urlGrabOpen(bufferd, index):
    global urlGrab, urlGrabSettings
    if bufferd == "":
        urlGrabPrint( "No current channel, you must specify one" )
    elif not urlGrab.hasBuffer(bufferd) :
        urlGrabPrint("No URL found - Invalid channel")
    else:
        if index <= 0:
            urlGrabPrint("No URL found - Invalid index")
            return
        url =  urlGrab.getUrl(bufferd,index)
        if url == "":
            urlGrabPrint("No URL found - Invalid index")
        else:
            urlGrabPrint("loading %s %sly" % (url, urlGrabSettings["method"]))
            urlGrabOpenUrl (url)

def urlGrabList( args ):
    global urlGrab
    if len(args) == 0:
        buf = hashBufferName(weechat.current_buffer())
    else:
        buf = args[0]
    if buf == "" or buf == "all":
        buf = "*"
    urlGrab.prnt(buf)


def urlGrabMain(data, bufferp, args):
    if args[0:2] == "**":
        keyEvent(data, bufferp, args[2:])
        return weechat.WEECHAT_RC_OK

    bufferd = hashBufferName(bufferp)
    largs = args.split(" ")
    #strip spaces
    while '' in largs:
        largs.remove('')
    while ' ' in largs:
        largs.remove(' ')
    if len(largs) == 0 or largs[0] == "show":
        if not urlgrab_buffer:
            init()
        refresh()
        weechat.buffer_set(urlgrab_buffer, "display", "1")
    elif largs[0] == 'open' and len(largs) == 2:
        urlGrabOpenUrl(largs[1])
    elif largs[0] == 'list':
        urlGrabList( largs[1:] )
    elif largs[0] == 'copy':
        if len(largs) > 1:
            no = int(largs[1])
            urlGrabCopy(bufferd, no)
        else:
            urlGrabCopy(bufferd,1)
    else:
        try:
            no = int(largs[0])
            if len(largs) > 1:
                urlGrabOpen(largs[1], no)
            else:
                urlGrabOpen(bufferd, no)
        except ValueError:
            #not a valid number so try opening it as a url..
            for url in urlRe.findall(largs[0]):
                urlGrabOpenUrl(stripParens(url))
            urlGrabPrint( "Unknown command '%s'.  Try '/help url' for usage" % largs[0])
    return weechat.WEECHAT_RC_OK

def buffer_input(*kwargs):
    return weechat.WEECHAT_RC_OK

def buffer_close(*kwargs):
    global urlgrab_buffer
    urlgrab_buffer =  None
    return weechat.WEECHAT_RC_OK

def keyEvent (data, bufferp, args):
    global urlGrab , urlGrabSettings, urlgrab_buffer, current_line
    if args == "refresh":
        refresh()
    elif args == "up":
        if current_line > 0:
            current_line = current_line -1
            refresh_line (current_line + 1)
            refresh_line (current_line)
            ugCheckLineOutsideWindow()
    elif args == "down":
         if current_line < len(urlGrab.globalUrls) - 1:
            current_line = current_line +1
            refresh_line (current_line - 1)
            refresh_line (current_line)
            ugCheckLineOutsideWindow()
    elif args == "scroll_top":
        temp_current = current_line
        current_line =  0
        refresh_line (temp_current)
        refresh_line (current_line)
        weechat.command(urlgrab_buffer, "/window scroll_top")
        pass
    elif args == "scroll_bottom":
        temp_current = current_line
        current_line =  len(urlGrab.globalUrls)
        refresh_line (temp_current)
        refresh_line (current_line)
        weechat.command(urlgrab_buffer, "/window scroll_bottom")
    elif args == "enter":
        if urlGrab.globalUrls[current_line]:
            urlGrabOpenUrl (urlGrab.globalUrls[current_line]['url'])

def refresh_line (y):
    global urlGrab , urlGrabSettings, urlgrab_buffer, current_line, max_buffer_length
    #Print format  Time(space)buffer(max4 spaces, but lined up)url
    format = "%%s%%s %%s%%-%ds%%s%%s" % (max_buffer_length+4)

    #non selected colors
    color_buffer = urlGrabSettings["color_buffer"]
    color_url = urlGrabSettings["color_url"]
    color_time =urlGrabSettings["color_time"]
    #selected colors
    color_buffer_selected = urlGrabSettings["color_buffer_selected"]
    color_url_selected = urlGrabSettings["color_url_selected"]
    color_time_selected = urlGrabSettings["color_time_selected"]

    color_bg_selected = urlGrabSettings["color_bg_selected"]

    color1 = color_time
    color2 = color_buffer
    color3 = color_url

    #If this line is selected we change the colors.
    if y == current_line:
          color1 = "%s,%s" % (color_time_selected, color_bg_selected)
          color2 = "%s,%s" % (color_buffer_selected, color_bg_selected)
          color3 = "%s,%s" % (color_url_selected, color_bg_selected)

    color1 = weechat.color(color1)
    color2 = weechat.color(color2)
    color3 = weechat.color(color3)
    text = format % (color1,
                    urlGrab.globalUrls[y]['time'],
                    color2,
                    urlGrab.globalUrls[y]['buffer'],
                    color3,
                    urlGrab.globalUrls[y]['url'] )
    weechat.prnt_y(urlgrab_buffer,y,text)

def ugCheckLineOutsideWindow():
    global urlGrab , urlGrabSettings, urlgrab_buffer, current_line, max_buffer_length
    if (urlgrab_buffer):
        infolist = weechat.infolist_get("window", "", "current")
        if (weechat.infolist_next(infolist)):
            start_line_y = weechat.infolist_integer(infolist, "start_line_y")
            chat_height = weechat.infolist_integer(infolist, "chat_height")
            if(start_line_y > current_line):
                weechat.command(urlgrab_buffer, "/window scroll -%i" %(start_line_y - current_line))
            elif(start_line_y <= current_line - chat_height):
                weechat.command(urlgrab_buffer, "/window scroll +%i"%(current_line - start_line_y - chat_height + 1))
        weechat.infolist_free(infolist)


def refresh():
    global urlGrab
    y=0
    for x in urlGrab.globalUrls:
        refresh_line (y)
        y += 1


def init():
    global urlGrab , urlGrabSettings, urlgrab_buffer
    if not urlgrab_buffer:
        urlgrab_buffer = weechat.buffer_new("urlgrab", "buffer_input", "", "buffer_close", "")
    if urlgrab_buffer:
        weechat.buffer_set(urlgrab_buffer, "type", "free")
        weechat.buffer_set(urlgrab_buffer, "key_bind_ctrl-R",        "/url **refresh")
        weechat.buffer_set(urlgrab_buffer, "key_bind_meta2-A",       "/url **up")
        weechat.buffer_set(urlgrab_buffer, "key_bind_meta2-B",       "/url **down")
        weechat.buffer_set(urlgrab_buffer, "key_bind_meta-ctrl-J",   "/url **enter")
        weechat.buffer_set(urlgrab_buffer, "key_bind_meta-ctrl-M",   "/url **enter")
        weechat.buffer_set(urlgrab_buffer, "key_bind_meta-meta2-1./~", "/url **scroll_top")
        weechat.buffer_set(urlgrab_buffer, "key_bind_meta-meta2-4~", "/url **scroll_bottom")
        weechat.buffer_set(urlgrab_buffer, "title","Lists the urls in the applications")
        weechat.buffer_set(urlgrab_buffer, "display", "1")

def completion_urls_cb(data, completion_item, bufferp, completion):
    """ Complete with URLS, for command '/url'. """
    global urlGrab
    bufferd = hashBufferName( bufferp)
    for url in urlGrab.globalUrls :
        if url['buffer'] == bufferd:
            weechat.hook_completion_list_add(completion, url['url'], 0, weechat.WEECHAT_LIST_POS_SORT)
    return weechat.WEECHAT_RC_OK

def ug_unload_script():
    """ Function called when script is unloaded. """
    global urlGrabSettings
    weechat.config_write(urlGrabSettings.config_file)
    return weechat.WEECHAT_RC_OK

#Main stuff
if ( import_ok and
    weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION,
        SCRIPT_LICENSE,SCRIPT_DESC, "ug_unload_script", "") ):
    urlgrab_buffer = None
    current_line = 0
    max_buffer_length = 0
    urlGrabSettings = UrlGrabSettings()
    urlGrab = UrlGrabber( urlGrabSettings['historysize'])
    weechat.hook_print("", "", "", 1, "urlGrabCheck", "")
    weechat.hook_command(SCRIPT_COMMAND,
                             "Url Grabber",
                             "[open <url> | <url> | show | copy [n] | [n] | list]",
                             "open or <url>: opens the url\n"
                             "show: Opens the select buffer to allow for url selection\n"
                             "copy: Copies the nth url to the system clipboard\n"
                             "list: Lists the urls in the current buffer\n",
                             "open %(urlgrab_urls) || %(urlgrab_urls) || "
                             "copy || show || list",
                             "urlGrabMain", "")
    weechat.hook_completion("urlgrab_urls", "list of URLs",
                                "completion_urls_cb", "")
else:
    print("failed to load weechat")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2011-2018 Sébastien Helleu <flashcode@flashtux.org>
# Copyright (C) 2012 ArZa <arza@arza.us>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# Contextual command line help for WeeChat.
# (this script requires WeeChat 0.3.5 or newer)
#
# History:
#
# 2018-04-10, Sébastien Helleu <flashcode@flashtux.org>:
#     version 0.5: fix infolist_time for WeeChat >= 2.2 (WeeChat returns a long
#                  integer instead of a string), fix PEP8 errors
# 2012-01-04, ArZa <arza@arza.us>:
#     version 0.4: settings for right align and space before help
# 2012-01-03, Sébastien Helleu <flashcode@flashtux.org>:
#     version 0.3: make script compatible with Python 3.x
# 2011-05-18, Sébastien Helleu <flashcode@flashtux.org>:
#     version 0.2: add options for aliases, start on load, list of commands to
#                  ignore; add default value in help of script options
# 2011-05-15, Sébastien Helleu <flashcode@flashtux.org>:
#     version 0.1: initial release
#

SCRIPT_NAME = 'cmd_help'
SCRIPT_AUTHOR = 'Sébastien Helleu <flashcode@flashtux.org>'
SCRIPT_VERSION = '0.5'
SCRIPT_LICENSE = 'GPL3'
SCRIPT_DESC = 'Contextual command line help'

SCRIPT_COMMAND = 'cmd_help'

import_ok = True

try:
    import weechat
except ImportError:
    print('This script must be run under WeeChat.')
    print('Get WeeChat now at: http://www.weechat.org/')
    import_ok = False

try:
    import re
    import time
except ImportError as message:
    print('Missing package(s) for %s: %s' % (SCRIPT_NAME, message))
    import_ok = False

cmdhelp_hooks = {
    'modifier': '',
    'timer': '',
    'command_run': '',
}
cmdhelp_option_infolist = ''
cmdhelp_option_infolist_fields = {}

# script options
cmdhelp_settings_default = {
    'display_no_help': [
        'on',
        'display "No help" when command is not found',
    ],
    'start_on_load': [
        'off',
        'auto start help when script is loaded',
    ],
    'stop_on_enter': [
        'on',
        'enter key stop help',
    ],
    'timer': [
        '0',
        ('number of seconds help is displayed (0 = display until help is '
         'toggled)'),
    ],
    'prefix': [
        '[',
        'string displayed before help',
    ],
    'suffix': [
        ']',
        'string displayed after help',
    ],
    'format_option': [
        '(${white:type}) ${description_nls}',
        ('format of help for options: free text with identifiers using '
         'format: ${name} or ${color:name}: color is a WeeChat '
         'color (optional), name is a field of infolist "option"'),
    ],
    'max_options': [
        '5',
        'max number of options displayed in list',
    ],
    'ignore_commands': [
        'map,me,die,restart',
        'comma-separated list of commands (without leading "/") to ignore',
    ],
    'color_alias': [
        'white',
        'color for text "Alias"',
    ],
    'color_alias_name': [
        'green',
        'color for alias name',
    ],
    'color_alias_value': [
        'green',
        'color for alias value',
    ],
    'color_delimiters': [
        'lightgreen',
        'color for delimiters',
    ],
    'color_no_help': [
        'red',
        'color for text "No help"',
    ],
    'color_list_count': [
        'white',
        'color for number of commands/options in list found',
    ],
    'color_list': [
        'green',
        'color for list of commands/options',
    ],
    'color_arguments': [
        'cyan',
        'color for command arguments',
    ],
    'color_option_name': [
        'yellow',
        'color for name of option found (by adding "*" to option name)',
    ],
    'color_option_help': [
        'brown',
        'color for help on option',
    ],
    'right_align': [
        'off',
        'align help to right',
    ],
    'right_padding': [
        '15',
        'padding to right when aligned to right',
    ],
    'space': [
        '2',
        'minimum space before help',
    ],
}
cmdhelp_settings = {}


def unhook(hooks):
    """Unhook something hooked by this script."""
    global cmdhelp_hooks
    for hook in hooks:
        if cmdhelp_hooks[hook]:
            weechat.unhook(cmdhelp_hooks[hook])
            cmdhelp_hooks[hook] = ''


def config_cb(data, option, value):
    """Called when a script option is changed."""
    global cmdhelp_settings, cmdhelp_hooks
    pos = option.rfind('.')
    if pos > 0:
        name = option[pos+1:]
        if name in cmdhelp_settings:
            cmdhelp_settings[name] = value
            if name == 'stop_on_enter':
                if value == 'on' and not cmdhelp_hooks['command_run']:
                    cmdhelp_hooks['command_run'] = weechat.hook_command_run(
                        '/input return', 'command_run_cb', '')
                elif value != 'on' and cmdhelp_hooks['command_run']:
                    unhook(('command_run',))
    return weechat.WEECHAT_RC_OK


def command_run_cb(data, buffer, command):
    """Callback for "command_run" hook."""
    global cmdhelp_hooks, cmdhelp_settings
    if cmdhelp_hooks['modifier'] and cmdhelp_settings['stop_on_enter'] == 'on':
        unhook(('timer', 'modifier'))
    return weechat.WEECHAT_RC_OK


def format_option(match):
    """Replace ${xxx} by its value in option format."""
    global cmdhelp_settings, cmdhelp_option_infolist
    global cmdhelp_option_infolist_fields
    string = match.group()
    end = string.find('}')
    if end < 0:
        return string
    field = string[2:end]
    color1 = ''
    color2 = ''
    pos = field.find(':')
    if pos:
        color1 = field[0:pos]
        field = field[pos+1:]
    if color1:
        color1 = weechat.color(color1)
        color2 = weechat.color(cmdhelp_settings['color_option_help'])
    fieldtype = cmdhelp_option_infolist_fields.get(field, '')
    if fieldtype == 'i':
        string = str(weechat.infolist_integer(cmdhelp_option_infolist, field))
    elif fieldtype == 's':
        string = weechat.infolist_string(cmdhelp_option_infolist, field)
    elif fieldtype == 'p':
        string = weechat.infolist_pointer(cmdhelp_option_infolist, field)
    elif fieldtype == 't':
        date = weechat.infolist_time(cmdhelp_option_infolist, field)
        # since WeeChat 2.2, infolist_time returns a long integer instead of
        # a string
        if not isinstance(date, str):
            date = time.strftime('%F %T', time.localtime(int(date)))
        string = date
    return '%s%s%s' % (color1, string, color2)


def get_option_list_and_desc(option, displayname):
    """Get list of options and description for option(s)."""
    global cmdhelp_settings, cmdhelp_option_infolist
    global cmdhelp_option_infolist_fields
    options = []
    description = ''
    cmdhelp_option_infolist = weechat.infolist_get('option', '', option)
    if cmdhelp_option_infolist:
        cmdhelp_option_infolist_fields = {}
        while weechat.infolist_next(cmdhelp_option_infolist):
            options.append(weechat.infolist_string(cmdhelp_option_infolist,
                                                   'full_name'))
            if not description:
                fields = weechat.infolist_fields(cmdhelp_option_infolist)
                for field in fields.split(','):
                    items = field.split(':', 1)
                    if len(items) == 2:
                        cmdhelp_option_infolist_fields[items[1]] = items[0]
                description = re.compile(r'\$\{[^\}]+\}').sub(
                    format_option, cmdhelp_settings['format_option'])
                if displayname:
                    description = '%s%s%s: %s' % (
                        weechat.color(cmdhelp_settings['color_option_name']),
                        weechat.infolist_string(cmdhelp_option_infolist,
                                                'full_name'),
                        weechat.color(cmdhelp_settings['color_option_help']),
                        description)
        weechat.infolist_free(cmdhelp_option_infolist)
        cmdhelp_option_infolist = ''
        cmdhelp_option_infolist_fields = {}
    return options, description


def get_help_option(input_args):
    """Get help about option or values authorized for option."""
    global cmdhelp_settings, cmdhelp_option_infolist
    global cmdhelp_option_infolist_fields
    pos = input_args.find(' ')
    if pos > 0:
        option = input_args[0:pos]
    else:
        option = input_args
    options, description = get_option_list_and_desc(option, False)
    if not options and not description:
        options, description = get_option_list_and_desc('%s*' % option, True)
    if len(options) > 1:
        try:
            max_options = int(cmdhelp_settings['max_options'])
        except ValueError:
            max_options = 5
        if len(options) > max_options:
            text = '%s...' % ', '.join(options[0:max_options])
        else:
            text = ', '.join(options)
        return '%s%d options: %s%s' % (
            weechat.color(cmdhelp_settings['color_list_count']),
            len(options),
            weechat.color(cmdhelp_settings['color_list']),
            text)
    if description:
        return '%s%s' % (weechat.color(cmdhelp_settings['color_option_help']),
                         description)
    return '%sNo help for option %s' % (
        weechat.color(cmdhelp_settings['color_no_help']), option)


def get_command_arguments(input_args, cmd_args):
    """Get command arguments according to command arguments given in input."""
    partial = ''
    input_firstarg = input_args.split(' ', 1)[0].lower()
    items = cmd_args.split('||')
    for item in items:
        item = item.strip()
        firstword = item.split(' ')[0]
        items2 = firstword.split('|')
        for item2 in items2:
            item2 = item2.strip().lower()
            if item2 == input_firstarg:
                return item
            if not partial and item2.startswith(input_firstarg):
                partial = item
    if partial:
        return partial
    return cmd_args


def get_help_command(plugin, input_cmd, input_args):
    """Get help for command in input."""
    global cmdhelp_settings
    if input_cmd == 'set' and input_args:
        return get_help_option(input_args)
    infolist = weechat.infolist_get('hook', '', 'command,%s' % input_cmd)
    cmd_plugin_name = ''
    cmd_command = ''
    cmd_args = ''
    cmd_desc = ''
    while weechat.infolist_next(infolist):
        cmd_plugin_name = (weechat.infolist_string(infolist, 'plugin_name') or
                           'core')
        cmd_command = weechat.infolist_string(infolist, 'command')
        cmd_args = weechat.infolist_string(infolist, 'args_nls')
        cmd_desc = weechat.infolist_string(infolist, 'description')
        if weechat.infolist_pointer(infolist, 'plugin') == plugin:
            break
    weechat.infolist_free(infolist)
    if cmd_plugin_name == 'alias':
        return '%sAlias %s%s%s => %s%s' % (
            weechat.color(cmdhelp_settings['color_alias']),
            weechat.color(cmdhelp_settings['color_alias_name']),
            cmd_command,
            weechat.color(cmdhelp_settings['color_alias']),
            weechat.color(cmdhelp_settings['color_alias_value']),
            cmd_desc,
        )
    if input_args:
        cmd_args = get_command_arguments(input_args, cmd_args)
    if not cmd_args:
        return None
    return '%s%s' % (weechat.color(cmdhelp_settings['color_arguments']),
                     cmd_args)


def get_list_commands(plugin, input_cmd, input_args):
    """Get list of commands (beginning with current input)."""
    global cmdhelp_settings
    infolist = weechat.infolist_get('hook', '', 'command,%s*' % input_cmd)
    commands = []
    plugin_names = []
    while weechat.infolist_next(infolist):
        commands.append(weechat.infolist_string(infolist, 'command'))
        plugin_names.append(
            weechat.infolist_string(infolist, 'plugin_name') or 'core')
    weechat.infolist_free(infolist)
    if commands:
        if len(commands) > 1 or commands[0].lower() != input_cmd.lower():
            commands2 = []
            for index, command in enumerate(commands):
                if commands.count(command) > 1:
                    commands2.append('%s(%s)' % (command, plugin_names[index]))
                else:
                    commands2.append(command)
            return '%s%d commands: %s%s' % (
                weechat.color(cmdhelp_settings['color_list_count']),
                len(commands2),
                weechat.color(cmdhelp_settings['color_list']),
                ', '.join(commands2))
    return None


def input_modifier_cb(data, modifier, modifier_data, string):
    """Modifier that will add help on command line (for display only)."""
    global cmdhelp_settings
    line = weechat.string_remove_color(string, '')
    if line == '':
        return string
    command = ''
    arguments = ''
    if weechat.string_input_for_buffer(line) != '':
        return string
    items = line.split(' ', 1)
    if len(items) > 0:
        command = items[0]
        if len(command) < 2:
            return string
        if len(items) > 1:
            arguments = items[1]
    if command[1:].lower() in cmdhelp_settings['ignore_commands'].split(','):
        return string
    current_buffer = weechat.current_buffer()
    current_window = weechat.current_window()
    plugin = weechat.buffer_get_pointer(current_buffer, 'plugin')
    msg_help = (get_help_command(plugin, command[1:], arguments) or
                get_list_commands(plugin, command[1:], arguments))
    if not msg_help:
        if cmdhelp_settings['display_no_help'] != 'on':
            return string
        msg_help = weechat.color(cmdhelp_settings['color_no_help'])
        if command:
            msg_help += 'No help for command %s' % command
        else:
            msg_help += 'No help'

    if cmdhelp_settings['right_align'] == 'on':
        win_width = weechat.window_get_integer(current_window, 'win_width')
        input_length = weechat.buffer_get_integer(current_buffer,
                                                  'input_length')
        help_length = len(weechat.string_remove_color(msg_help, ""))
        min_space = int(cmdhelp_settings['space'])
        padding = int(cmdhelp_settings['right_padding'])
        space = win_width - input_length - help_length - padding
        if space < min_space:
            space = min_space
    else:
        space = int(cmdhelp_settings['space'])

    color_delimiters = cmdhelp_settings['color_delimiters']
    return '%s%s%s%s%s%s%s' % (string,
                               space * ' ',
                               weechat.color(color_delimiters),
                               cmdhelp_settings['prefix'],
                               msg_help,
                               weechat.color(color_delimiters),
                               cmdhelp_settings['suffix'])


def timer_cb(data, remaining_calls):
    """Timer callback."""
    global cmdhelp_hooks
    if cmdhelp_hooks['modifier']:
        unhook(('modifier',))
        weechat.bar_item_update('input_text')
    return weechat.WEECHAT_RC_OK


def cmd_help_toggle():
    """Toggle help on/off."""
    global cmdhelp_hooks, cmdhelp_settings
    if cmdhelp_hooks['modifier']:
        unhook(('timer', 'modifier'))
    else:
        cmdhelp_hooks['modifier'] = weechat.hook_modifier(
            'input_text_display_with_cursor', 'input_modifier_cb', '')
        timer = cmdhelp_settings['timer']
        if timer and timer != '0':
            try:
                value = float(timer)
                if value > 0:
                    weechat.hook_timer(value * 1000, 0, 1, 'timer_cb', '')
            except ValueError:
                pass
    weechat.bar_item_update('input_text')


def cmd_help_cb(data, buffer, args):
    """Callback for /cmd_help command."""
    cmd_help_toggle()
    return weechat.WEECHAT_RC_OK


if __name__ == '__main__' and import_ok:
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION,
                        SCRIPT_LICENSE, SCRIPT_DESC, '', ''):
        # set allowed fields in option "format_option"
        fields = []
        infolist = weechat.infolist_get('option', '', 'weechat.plugin.*')
        if infolist:
            if weechat.infolist_next(infolist):
                strfields = weechat.infolist_fields(infolist)
                for field in strfields.split(','):
                    items = field.split(':', 1)
                    if len(items) == 2:
                        fields.append(items[1])
            weechat.infolist_free(infolist)
        if fields:
            cmdhelp_settings_default['format_option'][1] += (
                ': %s' % ', '.join(fields))

        # set default settings
        version = weechat.info_get("version_number", "") or 0
        for option, value in cmdhelp_settings_default.items():
            if weechat.config_is_set_plugin(option):
                cmdhelp_settings[option] = weechat.config_get_plugin(option)
            else:
                weechat.config_set_plugin(option, value[0])
                cmdhelp_settings[option] = value[0]
            if int(version) >= 0x00030500:
                weechat.config_set_desc_plugin(
                    option,
                    '%s (default: "%s")' % (value[1], value[0]))

        # detect config changes
        weechat.hook_config('plugins.var.python.%s.*' % SCRIPT_NAME,
                            'config_cb', '')

        # add hook to catch "enter" key
        if cmdhelp_settings['stop_on_enter'] == 'on':
            cmdhelp_hooks['command_run'] = weechat.hook_command_run(
                '/input return', 'command_run_cb', '')

        # add command
        weechat.hook_command(
            SCRIPT_COMMAND,
            'Contextual command line help.',
            '',
            'This comand toggles help on command line.\n\n'
            'It is recommended to bind this command on a key, for example '
            'F1:\n'
            '  /key bind <press alt-k> <press F1> /cmd_help\n'
            'which will give, according to your terminal something like:\n'
            '      /key bind meta-OP /cmd_help\n'
            '    or:\n'
            '      /key bind meta2-11~ /cmd_help\n\n'
            'To try: type "/server" (without pressing enter) and press F1 '
            '(then you can add arguments and enjoy dynamic help!)',
            '', 'cmd_help_cb', '')

        # auto start help
        if cmdhelp_settings['start_on_load'] == 'on':
            cmd_help_toggle()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2017 by nils_2 <weechatter@arcor.de>
# Copyright (c) 2015 by Damien Bargiacchi <icymidnight@gmail.com>
#
# stick buffer to a window, irssi like
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# idea by shad0VV@freenode.#weechat
#
# 2017-12-14: Sébastien Helleu <flashcode@flashtux.org>
#       0.6 : rename command "/autosetbuffer" by "/buffer_autoset" in example
#
# 2017-04-02: nils_2, (freenode.#weechat)
#       0.5 : support of "/input jump_smart" and "/buffer +/-" (reported: squigz)
#
# 2017-03-25: nils_2, (freenode.#weechat)
#       0.4 : script did not work with /go script and buffer names (reported: squigz)
#
# 2015-05-12: Damien Bargiacchi <icymidnight@gmail.com>
#       0.3 : Stop script from truncating localvar lookup to first character of the buffer number
#           : Clean up destination buffer number logic
#
# 2013-01-25: nils_2, (freenode.#weechat)
#       0.2 : make script compatible with Python 3.x
#           : smaller improvements
#
# 2013-01-21: nils_2, (freenode.#weechat)
#       0.1 : initial release
#
# requires: WeeChat version 0.3.6
#
# Development is currently hosted at
# https://github.com/weechatter/weechat-scripts

try:
    import weechat, sys

except Exception:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: http://www.weechat.org/")
    quit()

SCRIPT_NAME     = "stick_buffer"
SCRIPT_AUTHOR   = "nils_2 <weechatter@arcor.de>"
SCRIPT_VERSION  = "0.6"
SCRIPT_LICENSE  = "GPL"
SCRIPT_DESC     = "Stick buffers to particular windows, like irssi"

# ======================================[      config      ]====================================== #
SW_CONFIG_DEFAULTS = {
    'default_stick_window' : ('', 'The default window to stick a buffer to if no localvar '
                                  'stick_buffer_to_window is set'),
}

sw_config = {}

def init_config():
    for option, (default_value, description) in SW_CONFIG_DEFAULTS.items():
        if not weechat.config_is_set_plugin(option):
            weechat.config_set_plugin(option, default_value)
            sw_config[option] = default_value
        else:
            sw_config[option] = weechat.config_get_plugin(option)
        weechat.config_set_desc_plugin(option, '%s (default: "%s")' % (description, default_value))

    weechat.hook_config('plugins.var.python.' + SCRIPT_NAME + '.*', 'update_config', '')


def update_config(pointer, name, value):
    global sw_config
    option = name[len('plugins.var.python.' + SCRIPT_NAME + '.'):]
    sw_config[option] = value
    return weechat.WEECHAT_RC_OK

def get_default_stick_window_number():
    if sw_config['default_stick_window'] and sw_config['default_stick_window'].isdigit():
        return int(sw_config['default_stick_window'])
    return None

# ======================================[   buffer utils   ]====================================== #
def infolist_get_buffer_name_and_ptr_by_number(str_buffer_number):
    infolist = weechat.infolist_get('buffer', '', '')
    full_name = ''
    ptr_buffer = ''
    if infolist:
        while weechat.infolist_next(infolist):
            if int(str_buffer_number) == weechat.infolist_integer(infolist, 'number'):
                full_name = weechat.infolist_string(infolist, 'full_name')
                ptr_buffer = weechat.infolist_pointer(infolist, 'pointer')
                break
        weechat.infolist_free(infolist)
    return full_name, ptr_buffer

def infolist_get_buffer_name_and_ptr_by_name(str_buffer_name):
    infolist = weechat.infolist_get('buffer', '', '*%s*' % str_buffer_name)
    full_name = ''
    ptr_buffer = ''
    if infolist:
        while weechat.infolist_next(infolist):
            full_name = weechat.infolist_string(infolist, 'full_name')
            ptr_buffer = weechat.infolist_pointer(infolist, 'pointer')
            break
        weechat.infolist_free(infolist)
    return full_name, ptr_buffer

def infolist_get_first_entry_from_hotlist():
    infolist = weechat.infolist_get('hotlist', '', '')
    if infolist:
        weechat.infolist_next(infolist)         # go to first entry in hotlist
        buffer_name = weechat.infolist_string(infolist, 'buffer_name')
        buffer_number = weechat.infolist_integer(infolist, 'buffer_number')
        ptr_buffer = weechat.infolist_pointer(infolist, 'buffer_pointer')
        weechat.infolist_free(infolist)
    return buffer_name, ptr_buffer, buffer_number

def get_current_buffer_number():
    ptr_buffer = weechat.window_get_pointer(weechat.current_window(), 'buffer')
    return weechat.buffer_get_integer(ptr_buffer, 'number')

def get_destination_buffer_number(arg):
    mod = None
    num_str = arg
    if arg[0] in '+-*':
        num_str = arg[1:]
        mod = arg[0]
    if not num_str.isdigit():
        return None
    num = int(num_str)

    if not mod or mod == '*':
        return num

    current_buffer = get_current_buffer_number()
    if not current_buffer:
        return None
    if mod == '+':
        return current_buffer + num
    else: # mod == '-'
        return current_buffer - num

# ======================================[    callbacks     ]====================================== #
def buffer_switch_cb(data, buffer, command):
#    weechat.prnt("","data: %s   buffer: %s  command: %s" % (data,buffer,command))
    # command exist?
    if command == '':
        return weechat.WEECHAT_RC_OK

    # get command without leading command char!
    cmd = command[1:].strip().split(' ',)[0:1]
    # get number from command /buffer
    args = command.strip().split(' ',)[1:]
    ptr_buffer = ''

    if "input" in cmd and "jump_smart" in args:
        buffer_name, ptr_buffer, buffer_number = infolist_get_first_entry_from_hotlist()

    if "buffer" in cmd:
        if len(args) != 1:
            return weechat.WEECHAT_RC_OK

        # check if argument is a buffer "number"
        destination_buffer = get_destination_buffer_number(args[0])
        if destination_buffer:
            if destination_buffer < 1:
                destination_buffer = 1
            buffer_name, ptr_buffer = infolist_get_buffer_name_and_ptr_by_number(destination_buffer)
        else:
            # search for buffer name
            buffer_name, ptr_buffer = infolist_get_buffer_name_and_ptr_by_name(args[0])

    if not ptr_buffer:
        return weechat.WEECHAT_RC_OK

    if ptr_buffer == weechat.window_get_pointer(weechat.current_window(), 'buffer'):
        return weechat.WEECHAT_RC_OK
    window_number = weechat.buffer_get_string(ptr_buffer, 'localvar_stick_buffer_to_window')
    if not window_number:
        window_number = get_default_stick_window_number()
    if window_number:
        weechat.command('', '/window %s' % window_number)
        weechat.command('', '/buffer %s' % buffer_name)
        return weechat.WEECHAT_RC_OK_EAT
    else:
        return weechat.WEECHAT_RC_OK

def cmd_cb(data, buffer, args):
    args = args.strip().lower().split(' ')

    if args[0] == 'list':
        weechat.command('', '/set *.localvar_set_stick_buffer_to_window')
    elif args[0] in ['', 'help']:
        show_help()
    else:
        print_error('Unrecognized command %s\n' % ' '.join(args))
        show_help()

    return weechat.WEECHAT_RC_OK

# ======================================[       util       ]====================================== #

def show_help():
    weechat.command('', '/help %s' % SCRIPT_NAME)

def print_error(message):
    weechat.prnt('', '%s%s: %s' % (weechat.prefix("error"), SCRIPT_NAME, message))

# ======================================[       main       ]====================================== #

def main():
        version = weechat.info_get('version_number', '') or 0

        if int(version) < 0x00030600:
            print_error('script needs version 0.3.6 or higher')
            weechat.command('', "/wait 1ms /python unload %s" % SCRIPT_NAME)
            return

        init_config()

        description = """
{script_name} can make sure that when switching to a buffer it appears only in a particular window.
To trigger this behaviour set the localvar 'stick_buffer_to_window' to the desired window number.

You will need the script 'buffer_autoset.py' installed to make local variables persistent; see the
examples below.

Examples:
 Temporarily stick the current buffer to window 3:
   /buffer set localvar_set_stick_buffer_to_window 3
 Stick buffer #weechat to window 2:
   /buffer #weechat
   /buffer set localvar_set_stick_buffer_to_window 2
   /buffer_autoset add irc.freenode.#weechat stick_buffer_to_window 2
 Set the default stick-to window to window 5:
   /set plugins.var.python.{script_name}.default_stick_window 5
 List buffers with persistent stickiness:
   /{script_name} list
 Show this help:
   /{script_name} help
 Display local variables for current buffer:
   /buffer localvar
""".format(script_name = SCRIPT_NAME)

        weechat.hook_command(SCRIPT_NAME, SCRIPT_DESC, 'list', description, 'list %-', 'cmd_cb', '')

        weechat.hook_command_run('/buffer *', 'buffer_switch_cb', '')
        weechat.hook_command_run('/input jump_smart', 'buffer_switch_cb', '')

if __name__ == '__main__':
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC,
                        '', ''):
        main()
